@@ -1,4 +1,4 @@
-/* audio_FreeBSD.c - FreeBSD 2.0 / 2.0.5 / 2.1.0 audio stubs.  */
+/* $Id: audio_FreeBSD.c,v 1.3 1998/09/03 06:08:44 boyns Exp $ */
 
 /*
  * Copyright (C) 1993-98 Mark R. Boyns <boyns@doit.org>
@@ -231,6 +231,14 @@ rplay_audio_open (void)
     if ((rplay_audio_fd = open (rplay_audio_device, (O_WRONLY | O_NONBLOCK), 0)) < 0)
 	return -1;
 
+    if (fcntl (rplay_audio_fd, F_SETFD, 1) < 0)
+    {
+	report (REPORT_ERROR,
+		"rplay_audio_open: close-on-exec %d\n",
+		sys_err_str(errno));
+	/* return -1; */
+    }
+
     if (rplay_audio_init () < 0)
 	return -1;
 
