@@ -1,4 +1,4 @@
-/* audio_solaris.c - Solaris 2.x audio stubs.  */
+/* $Id: audio_solaris.c,v 1.3 1998/09/03 06:08:48 boyns Exp $ */
 
 /*
  * Copyright (C) 1993-98 Mark R. Boyns <boyns@doit.org>
@@ -168,6 +168,17 @@ rplay_audio_init ()
 	rplay_audio_port = optional_port ? optional_port : RPLAY_AUDIO_PORT_SPEAKER;
 	rplay_audio_table = amd_table;
     }
+    else if (strcmp (d.name, "SUNW,sb16") == 0)
+    {
+	report (REPORT_DEBUG, "%s device detected\n", d.name);
+	rplay_audio_sample_rate = optional_sample_rate ? optional_sample_rate : 44100;
+	rplay_audio_precision = optional_precision ? optional_precision : 16;
+	rplay_audio_channels = optional_channels ? optional_channels : 2;
+	rplay_audio_format = optional_format ? optional_format :
+	    rplay_audio_precision == 16 ? RPLAY_FORMAT_LINEAR_16 : RPLAY_FORMAT_LINEAR_8;
+	rplay_audio_port = optional_port ? optional_port : RPLAY_AUDIO_PORT_LINEOUT | RPLAY_AUDIO_PORT_SPEAKER;
+	rplay_audio_table = dbri_table; /* use the dbri table */
+    }
     else
     {
 	report (REPORT_ERROR, "`%s' unknown audio device detected\n", d.name);
@@ -283,6 +294,14 @@ rplay_audio_open ()
 	return -1;
     }
 
+    if (fcntl (rplay_audio_fd, F_SETFD, 1) < 0)
+    {
+	report (REPORT_ERROR,
+		"rplay_audio_open: close-on-exec %d\n",
+		sys_err_str(errno));
+	/* return -1; */
+    }
+
     if (rplay_audio_init () < 0)
     {
 	return -1;
