@@ -1,4 +1,4 @@
-/* sound.c - Sound file reading, parsing, and caching.  */
+/* $Id: sound.c,v 1.6 1998/10/15 15:15:25 boyns Exp $ */
 
 /*
  * Copyright (C) 1993-98 Mark R. Boyns <boyns@doit.org>
@@ -20,8 +20,6 @@
  * Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
-
-
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -531,6 +529,7 @@ sound_create ()
     s->offset = 0;
     s->size = 0;
     s->tail = 0;
+    s->chunk_size = 0;
     s->mapped = 0;
     s->cache = NULL;
     s->flow = NULL;
@@ -912,6 +911,15 @@ sound_map (s)
 	p += 4;
 
 	s->offset = ulaw_hdr_size;
+
+	if (data_size == ~0)
+	{
+	    sound_chunk_size = s->size - s->offset;
+	}
+	else
+	{
+	    sound_chunk_size = data_size;
+	}
     }
     /* aiff */
     else if (strncmp (buf, "FORM", 4) == 0)
@@ -1203,6 +1211,8 @@ sound_map (s)
 
 	    if (block_id == 1)	/* VOC_DATA */
 	    {
+		//sound_chunk_size = block_length;
+
 		rate = *p++;
 		s->input_precision = 8;
 		s->output_precision = 8;
@@ -1212,6 +1222,8 @@ sound_map (s)
 	    }
 	    else if (block_id == 9) /* VOC_DATA_16 */
 	    {
+		//sound_chunk_size = block_length;
+
 		s->sample_rate = little_long (p);
 		p += 4;
 
@@ -1236,7 +1248,7 @@ sound_map (s)
 		
 		s->channels = *p++;
 		
-		p++; /* uknown */
+		p++; /* unknown */
 		p++; /* not used */
 		p++; /* not used */
 		p++; /* not used */
@@ -1413,23 +1425,36 @@ sound_map (s)
 
     /* The input sample size is a floating point number since compressed
        samples can be smaller than 8 bits.  */
-    s->input_sample_size = (s->input_precision / 8) * s->channels; /* XXX output_precision -> input_precision */
+    /* XXX output_precision -> input_precision */
+    s->input_sample_size = (s->input_precision / 8) * s->channels;
     s->output_sample_size = (s->output_precision >> 3) * s->channels;
 
+    if (sound_chunk_size > 0)
+    {
+	/* Make sure sound_chunk_size is valid */
+	if (s->size == 0 || sound_chunk_size < s->size) /* size==0 for flows */
+	{
+	    s->chunk_size = sound_chunk_size;
+	}
+    }
 
-    /* Calculate the tail (which can be zero).  Some wav files have
-       weird sound_chunk_sizes so make sure sound_chunk_size is
-       something valid. */
-    if (sound_chunk_size
-	&& sound_chunk_size > 0 && sound_chunk_size < s->size)
+    /* Calculate tail only if chunk_size is in sound header */
+    if (s->chunk_size)
     {
-	s->tail = s->offset + sound_chunk_size;
+	s->tail = s->offset + s->chunk_size;
     }
-    else
+
+    /* Calculate chunk_size */
+    if (!s->chunk_size && s->size)
     {
-	s->tail = 0;
+	s->chunk_size = s->size - s->offset;
     }
-	
+
+#if 0    
+    printf ("SOUND: %s size=%d chunk=%d tail=%d offset=%d\n",
+	    s->name, s->size, s->chunk_size, s->tail, s->offset);
+#endif    
+
     if (s->type != SOUND_FILE)
     {
 	s->size = 0;
@@ -1445,7 +1470,7 @@ sound_map (s)
 #endif    
     else
     {
-	s->samples = NUMBER_OF_SAMPLES (s);
+	s->samples = number_of_samples (s, 0); // NUMBER_OF_SAMPLES (s);
     }
 
     s->mapped = 1;
@@ -1928,7 +1953,7 @@ sound_open (s, use_helper)
 
 	    if (s->type == SOUND_FILE)
 	    {
-		input_fd = open (s->path, O_RDONLY | O_NDELAY, 0);
+		input_fd = open (s->path, O_RDONLY /* | O_NDELAY */, 0);
 		if (input_fd < 0)
 		{
 		    report (REPORT_ERROR, "can't open %s: %s\n", s->path, sys_err_str (errno));
@@ -2009,7 +2034,8 @@ sound_open (s, use_helper)
 	    si->fd = open (s->path, O_RDONLY | O_NDELAY, 0);
 	    if (si->fd < 0)
 	    {
-		report (REPORT_DEBUG, "sound_open: open %s: %s\n", s->path, sys_err_str (errno));
+		report (REPORT_DEBUG, "sound_open: open %s: %s\n", s->path,
+			sys_err_str (errno));
 		sound_close (si);
 		return NULL;
 	    }
@@ -2742,3 +2768,46 @@ sound_pipe_read (si)
 }
 #endif
 
+#ifdef __STDC__
+static int
+bytes_in_sound (SOUND *s)
+#else
+static int
+bytes_in_sound (s)
+    SOUND *s;
+#endif
+{
+    int bytes = 0;
+    
+    if (s->chunk_size)
+    {
+	bytes = s->chunk_size;
+    }
+    else if (s->tail)
+    {
+	bytes = s->tail - s->offset;
+    }
+    else if (s->size)
+    {
+	bytes = s->size - s->offset;
+    }
+
+    return bytes;
+}
+
+#ifdef __STDC__
+int
+number_of_samples (SOUND *s, int bytes)
+#else
+int
+number_of_samples (s, bytes)
+    SOUND *s;
+    int bytes;
+#endif
+{
+    if (!bytes)
+    {
+	bytes = bytes_in_sound (s);
+    }
+    return (bytes << 3) / (s->input_precision * s->channels);
+}
