@@ -1,4 +1,4 @@
-/* spool.c - Audio spool maintenance.  */
+/* $Id: spool.c,v 1.2 1998/08/13 06:14:07 boyns Exp $ */
 
 /*
  * Copyright (C) 1993-98 Mark R. Boyns <boyns@doit.org>
@@ -755,7 +755,8 @@ spool_flow_insert (sp, b)
 	//s->size += tmp_buf->nbytes;
 	sp->si->water_mark += tmp_buf->nbytes;
     }
-    s->samples = NUMBER_OF_SAMPLES (s);
+
+    s->samples = number_of_samples (s, sp->si->water_mark);
 
     /* Wakeup the spool entry. */
     if (sp->state == SPOOL_WAIT)
